@@ -22,7 +22,7 @@ $VERSION = '1.14';
 use Image::ExifTool qw(:Utils);
 use Image::ExifTool::Exif;
 
-# EXIF table tag ID's which are part of the EXIF 2.31 specification
+# EXIF table tag ID's which are part of the EXIF 2.32 specification
 # (also used by BuildTagLookup to add underlines in HTML version of EXIF Tag Table)
 %exifSpec = (
     0x1 => 1,
@@ -42,16 +42,16 @@ use Image::ExifTool::Exif;
     0x11a => 1,  0x8834 => 1,  0x9403 => 1,  0xa40a => 1,
     0x11b => 1,  0x8835 => 1,  0x9404 => 1,  0xa40b => 1,
     0x11c => 1,  0x9000 => 1,  0x9405 => 1,  0xa40c => 1,
-    0x128 => 1,  0x9003 => 1,  0xa000 => 1,  0xa420 => 1,
-    0x12d => 1,  0x9004 => 1,  0xa001 => 1,  0xa430 => 1,
-    0x131 => 1,  0x9010 => 1,  0xa002 => 1,  0xa431 => 1,
-    0x132 => 1,  0x9011 => 1,  0xa003 => 1,  0xa432 => 1,
-    0x13b => 1,  0x9012 => 1,  0xa004 => 1,  0xa433 => 1,
-    0x13e => 1,  0x9101 => 1,  0xa005 => 1,  0xa434 => 1,
-    0x13f => 1,  0x9102 => 1,  0xa20b => 1,  0xa435 => 1,
-    0x201 => 1,  0x9201 => 1,  0xa20c => 1,
-    0x202 => 1,  0x9202 => 1,  0xa20e => 1,
-    0x211 => 1,  0x9203 => 1,  0xa20f => 1,
+    0x128 => 1,  0x9003 => 1,  0xa000 => 1,  0xa460 => 1,
+    0x12d => 1,  0x9004 => 1,  0xa001 => 1,  0xa461 => 1,
+    0x131 => 1,  0x9010 => 1,  0xa002 => 1,  0xa462 => 1,
+    0x132 => 1,  0x9011 => 1,  0xa003 => 1,  0xa420 => 1,
+    0x13b => 1,  0x9012 => 1,  0xa004 => 1,  0xa430 => 1,
+    0x13e => 1,  0x9101 => 1,  0xa005 => 1,  0xa431 => 1,
+    0x13f => 1,  0x9102 => 1,  0xa20b => 1,  0xa432 => 1,
+    0x201 => 1,  0x9201 => 1,  0xa20c => 1,  0xa433 => 1,
+    0x202 => 1,  0x9202 => 1,  0xa20e => 1,  0xa434 => 1,
+    0x211 => 1,  0x9203 => 1,  0xa20f => 1,  0xa435 => 1,
     0x212 => 1,  0x9204 => 1,  0xa210 => 1,
     0x213 => 1,  0x9205 => 1,  0xa214 => 1,
     0x214 => 1,  0x9206 => 1,  0xa215 => 1,
@@ -253,7 +253,7 @@ my %validateInfo = (
     Groups => { 0 => 'ExifTool', 1 => 'ExifTool', 2 => 'ExifTool' },
     Notes => q{
         generated only if specifically requested.  Requesting this tag automatically
-        enables the L<API Validate option|../ExifTool.html#Validate>, imposing
+        enables the API L<Validate|../ExifTool.html#Validate> option, imposing
         additional validation checks when extracting metadata.  Returns the number
         of errors, warnings and minor warnings encountered.  Note that the Validate
         feature focuses mainly on validation of TIFF/EXIF metadata
@@ -540,7 +540,7 @@ sub FinishValidate($$)
                     } elsif ($result eq '') {
                         $pre = defined $val ? 'Invalid value for' : "Missing required $fileType";
                     } else {
-                        next if $result == '1';
+                        next if $result eq '1';
                         $pre = $result;
                     }
                 } else {
