@@ -89,7 +89,8 @@ my %ctboID = (
 );
 
 # mark UserData tags that don't have ItemList counterparts as Preferred
-# (and for now, set Writable to 0 for any tag with a RawConv)
+# - and set Preferred to 0 for any Avoid-ed tag
+# - also, for now, set Writable to 0 for any tag with a RawConv and no RawConvInv
 {
     my $itemList = \%Image::ExifTool::QuickTime::ItemList;
     my $userData = \%Image::ExifTool::QuickTime::UserData;
@@ -98,10 +99,12 @@ my %ctboID = (
         my $tagInfo = $$itemList{$tag};
         if (ref $tagInfo ne 'HASH') {
             next if ref $tagInfo;
-            $tagInfo = $$userData{$tag} = { Name => $tagInfo };
+            $tagInfo = $$itemList{$tag} = { Name => $tagInfo };
+        } else {
+            $$tagInfo{Writable} = 0 if $$tagInfo{RawConv} and not $$tagInfo{RawConvInv};
+            $$tagInfo{Avoid} and $$tagInfo{Preferred} = 0, next;
+            next if defined $$tagInfo{Preferred} and not $$tagInfo{Preferred};
         }
-        $$tagInfo{Writable} = 0 if $$tagInfo{RawConv};
-        next if $$tagInfo{Avoid} or defined $$tagInfo{Preferred} and not $$tagInfo{Preferred};
         $pref{$$tagInfo{Name}} = 1;
     }
     foreach $tag (TagTableKeys($userData)) {
@@ -109,9 +112,11 @@ my %ctboID = (
         if (ref $tagInfo ne 'HASH') {
             next if ref $tagInfo;
             $tagInfo = $$userData{$tag} = { Name => $tagInfo };
+        } else {
+            $$tagInfo{Writable} = 0 if $$tagInfo{RawConv} and not $$tagInfo{RawConvInv};
+            $$tagInfo{Avoid} and $$tagInfo{Preferred} = 0, next;
+            next if defined $$tagInfo{Preferred} or $pref{$$tagInfo{Name}};
         }
-        $$tagInfo{Writable} = 0 if $$tagInfo{RawConv};
-        next if $$tagInfo{Avoid} or defined $$tagInfo{Preferred} or $pref{$$tagInfo{Name}};
         $$tagInfo{Preferred} = 1;
     }
 }
@@ -365,12 +370,23 @@ sub WriteKeys($$$)
                 my $nvHash = $et->GetNewValueHash($tagInfo);
                 # drop this tag if it is being deleted
                 if ($nvHash and $et->IsOverwriting($nvHash) > 0 and not defined $et->GetNewValue($nvHash)) {
-                    # delete this key
-                    $et->VPrint(1, "$$et{INDENT}\[deleting Keys entry $index '${tag}']\n");
-                    $pos += $len;
-                    $remap{$index++} = 0;
-                    ++$$et{CHANGED};
-                    next;
+                    # don't delete this key if we could be writing any alternate-language version of this tag
+                    my ($t, $dontDelete);
+                    foreach $t (keys %$newTags) {
+                        next unless $$newTags{$t}{SrcTagInfo} and $$newTags{$t}{SrcTagInfo} eq $tagInfo;
+                        my $nv = $et->GetNewValueHash($$newTags{$t});
+                        next unless $et->IsOverwriting($nv) and defined $et->GetNewValue($nv);
+                        $dontDelete = 1;
+                        last;
+                    }
+                    unless ($dontDelete) {
+                        # delete this key
+                        $et->VPrint(1, "$$et{INDENT}\[deleting Keys entry $index '${tag}']\n");
+                        $pos += $len;
+                        $remap{$index++} = 0;
+                        ++$$et{CHANGED};
+                        next;
+                    }
                 }
             }
         }
@@ -393,7 +409,7 @@ sub WriteKeys($$$)
         next unless $$nvHash{IsCreating} and $et->IsOverwriting($nvHash) and
             defined $et->GetNewValue($nvHash);
         # add new entry to 'keys' data
-        my $val = "com.apple.quicktime.$id";
+        my $val = $id =~ /^com\./ ? $id : "com.apple.quicktime.$id";
         $newData .= Set32u(8 + length($val)) . 'mdta' . $val;
         $et->VPrint(1, "$$et{INDENT}\[adding Keys entry $newIndex '${id}']\n");
         $add{$newIndex++} = $tagInfo;
@@ -674,8 +690,8 @@ sub WriteQuickTime($$$)
     local $_;
     my ($et, $dirInfo, $tagTablePtr) = @_;
     $et or return 1;    # allow dummy access to autoload this package
-    my ($mdat, @mdat, @mdatEdit, $edit, $track, $outBuff, $co, $term, $err, $delCount);
-    my (%langTags, $canCreate, $delGrp, %boxPos, $createKeys, $newTags, %didDir, $writeLast);
+    my ($mdat, @mdat, @mdatEdit, $edit, $track, $outBuff, $co, $term, $delCount);
+    my (%langTags, $canCreate, $delGrp, %boxPos, %didDir, $writeLast, $err);
     my $outfile = $$dirInfo{OutFile} || return 0;
     my $raf = $$dirInfo{RAF};       # (will be null for lower-level atoms)
     my $dataPt = $$dirInfo{DataPt}; # (will be null for top-level atoms)
@@ -684,6 +700,8 @@ sub WriteQuickTime($$$)
     my $parent = $$dirInfo{Parent};
     my $addDirs = $$et{ADD_DIRS};
     my $didTag = $$et{DidTag};
+    my $newTags = { };
+    my $createKeys = 0;
     my ($rtnVal, $rtnErr) = $dataPt ? (undef, undef) : (1, 0);
 
     if ($dataPt) {
@@ -700,18 +718,15 @@ sub WriteQuickTime($$$)
     my $curPath = join '-', @{$$et{PATH}};
     my ($dir, $writePath) = ($dirName, $dirName);
     $writePath = "$dir-$writePath" while defined($dir = $$et{DirMap}{$dir});
-    # hack for Keys write path (its containing Meta is in a different location)
-    $createKeys = 1 if $$addDirs{Keys} and $curPath =~ /^MOV-Movie(-Meta(-ItemList)?)?$/;
-    if ($curPath eq $writePath or $createKeys) {
-        $canCreate = 1;
-        $delGrp = $$et{DEL_GROUP}{$dirName};
-    }
-    # edit existing Keys tags in ItemList if we are at the correct path
-    if ($curPath eq 'MOV-Movie-Meta-ItemList') {
-        $newTags = { };
+    # hack to create Keys directories if necessary (its containing Meta is in a different location)
+    if ($$addDirs{Keys} and $curPath =~ /^MOV-Movie(-Meta)?$/) {
+        $createKeys = 1;    # create new Keys directories
+    } elsif ($curPath eq 'MOV-Movie-Meta-ItemList') {
+        $createKeys = 2;    # create new Keys tags
         my $keys = $$et{Keys};
         if ($keys) {
             # add new tag entries for existing Keys tags, now that we know their ID's
+            # - first make lookup to convert Keys tagInfo ref to index number
             my ($index, %keysInfo);
             foreach $index (keys %{$$keys{Info}}) {
                 $keysInfo{$$keys{Info}{$index}} = $index if $$keys{Remap}{$index};
@@ -723,22 +738,30 @@ sub WriteQuickTime($$$)
                 $index = $keysInfo{$tagInfo} || ($$tagInfo{SrcTagInfo} and $keysInfo{$$tagInfo{SrcTagInfo}});
                 next unless $index;
                 my $id = Set32u($index);
-                $id .= '-' . $$tagInfo{LangCode} if $$tagInfo{LangCode};
+                if ($$tagInfo{LangCode}) {
+                    # add to lookup of language tags we are writing with this ID
+                    $langTags{$id} = { } unless $langTags{$id};
+                    $langTags{$id}{$_} = $tagInfo;
+                    $id .= '-' . $$tagInfo{LangCode};
+                }
                 $$newTags{$id} = $tagInfo;
             }
         }
     } else {
         # get hash of new tags to edit/create in this directory
         $newTags = $et->GetNewTagInfoHash($tagTablePtr);
+        # make lookup of language tags for each ID
+        foreach (keys %$newTags) {
+            next unless $$newTags{$_}{LangCode} and $$newTags{$_}{SrcTagInfo};
+            my $id = $$newTags{$_}{SrcTagInfo}{TagID};
+            $langTags{$id} = { } unless $langTags{$id};
+            $langTags{$id}{$_} = $$newTags{$_};
+        }
     }
-    # make lookup of language tags for each ID
-    foreach (keys %$newTags) {
-        next unless $$newTags{$_}{LangCode} and $$newTags{$_}{SrcTagInfo};
-        my $id = $$newTags{$_}{SrcTagInfo}{TagID};
-        $langTags{$id} = { } unless $langTags{$id};
-        $langTags{$id}{$_} = $$newTags{$_};
+    if ($curPath eq $writePath or $createKeys) {
+        $canCreate = 1;
+        $delGrp = $$et{DEL_GROUP}{$dirName};
     }
-
     for (;;) {      # loop through all atoms at this level
         my ($hdr, $buff, $keysIndex);
         my $n = $raf->Read($hdr, 8);
@@ -770,11 +793,18 @@ sub WriteQuickTime($$$)
             }
             $size = $hi * 4294967296 + $lo - 16;
             $size < 0 and $et->Error('Invalid extended atom size'), last;
-        } elsif ($size == -8 and not $dataPt) {
-            # size of zero is only valid for top-level atom, and
-            # indicates the atom extends to the end of file
-            # (save in mdat list to write later; with zero end position to copy rest of file)
-            push @mdat, [ $raf->Tell(), 0, $hdr ];
+        } elsif ($size == -8) {
+            if ($dataPt) {
+                my $pos = $raf->Tell() - 4;
+                $raf->Seek(0,2);
+                my $str = $$dirInfo{DirName} . ' with ' . ($raf->Tell() - $pos) . ' bytes';
+                $et->Error("Terminator found in $str remaining", 1);
+            } else {
+                # size of zero is only valid for top-level atom, and
+                # indicates the atom extends to the end of file
+                # (save in mdat list to write later; with zero end position to copy rest of file)
+                push @mdat, [ $raf->Tell(), 0, $hdr ];
+            }
             last;
         } elsif ($size < 0) {
             if ($$tagTablePtr{VARS}{IGNORE_BAD_ATOMS} and $dataPt) {
@@ -820,12 +850,15 @@ sub WriteQuickTime($$$)
 
         # if this atom stores offsets, save its location so we can fix up offsets later
         # (are there any other atoms that may store absolute file offsets?)
-        if ($tag =~ /^(stco|co64|iloc|mfra|gps |CTBO|uuid)$/) {
+        if ($tag =~ /^(stco|co64|iloc|mfra|moof|sidx|saio|gps |CTBO|uuid)$/) {
             # (note that we only need to do this if the movie data is stored in this file)
             my $flg = $$et{QtDataFlg};
-            if ($tag eq 'mfra') {
+            if ($tag eq 'mfra' or $tag eq 'moof') {
                 $et->Error("Can't yet handle movie fragments when writing");
                 return $rtnVal;
+            } elsif ($tag eq 'sidx' or $tag eq 'saio') {
+                $et->Error("Can't yet handle $tag box when writing");
+                return $rtnVal;
             } elsif ($tag eq 'iloc') {
                 Handle_iloc($et, $dirInfo, \$buff, $outfile) or $et->Error('Error parsing iloc atom');
             } elsif ($tag eq 'gps ') {
@@ -1003,7 +1036,7 @@ sub WriteQuickTime($$$)
                                         my $prVal = $newVal;
                                         my $flags = FormatQTValue($et, \$newVal, $$tagInfo{Format});
                                         next unless defined $newVal;
-                                        my ($ctry, $lang) = (0,0);
+                                        my ($ctry, $lang) = (0, $undLang);
                                         if ($$ti{LangCode}) {
                                             unless ($$ti{LangCode} =~ /^([A-Z]{3})?[-_]?([A-Z]{2})?$/i) {
                                                 $et->Warn("Invalid language code for $$ti{Name}");
@@ -1083,7 +1116,6 @@ sub WriteQuickTime($$$)
                                     my $grp = $et->GetGroup($langInfo, 1);
                                     $et->VerboseValue("- $grp:$$langInfo{Name}", $val);
                                     $et->VerboseValue("+ $grp:$$langInfo{Name}", $prVal);
-                                    $$didTag{$nvHash} = 1 if $nvHash;
                                     $newData = substr($buff, 0, $pos-16) unless defined $newData;
                                     $newData .= pack('Na4Nnn', length($newVal)+16, $type, $flags, $ctry, $lang);
                                     $newData .= $newVal;
@@ -1094,6 +1126,7 @@ sub WriteQuickTime($$$)
                             } elsif (defined $newData) {
                                 $newData .= substr($buff, $pos, $len);
                             }
+                            $$didTag{$nvHash} = 1 if $nvHash;
                         }
                         $newData .= substr($buff, $pos) if defined $newData and $pos < $size;
                         undef $val; # (already constructed $newData)
@@ -1136,24 +1169,31 @@ sub WriteQuickTime($$$)
                     } else {
                         $val = $buff;
                     }
-                    if ($nvHash and defined $val and $et->IsOverwriting($nvHash, $val)) {
-                        $newData = $et->GetNewValue($nvHash);
-                        $newData = '' unless defined $newData or $canCreate;
-                        ++$$et{CHANGED};
-                        my $grp = $et->GetGroup($langInfo, 1);
-                        $et->VerboseValue("- $grp:$$langInfo{Name}", $val);
-                        next unless defined $newData and not $$didTag{$nvHash};
-                        $et->VerboseValue("+ $grp:$$langInfo{Name}", $newData);
-                        $$didTag{$nvHash} = 1;   # set flag so we don't add this tag again
-                        # add back necessary header and encode as necessary
-                        if (defined $lang) {
-                            $newData = $et->Encode($newData, $lang < 0x400 ? $charsetQuickTime : 'UTF8');
-                            if ($$tagInfo{IText} and $$tagInfo{IText} == 6) {
-                                $newData = pack('Nn', 0, $lang) . $newData . "\0";
-                            } else {
-                                $newData = pack('nn', length($newData), $lang) . $newData;
+                    if ($nvHash and defined $val) {
+                        if ($et->IsOverwriting($nvHash, $val)) {
+                            $newData = $et->GetNewValue($nvHash);
+                            $newData = '' unless defined $newData or $canCreate;
+                            ++$$et{CHANGED};
+                            my $grp = $et->GetGroup($langInfo, 1);
+                            $et->VerboseValue("- $grp:$$langInfo{Name}", $val);
+                            next unless defined $newData and not $$didTag{$nvHash};
+                            $et->VerboseValue("+ $grp:$$langInfo{Name}", $newData);
+                            # add back necessary header and encode as necessary
+                            if (defined $lang) {
+                                $newData = $et->Encode($newData, $lang < 0x400 ? $charsetQuickTime : 'UTF8');
+                                if ($$tagInfo{IText} and $$tagInfo{IText} == 6) {
+                                    $newData = pack('Nn', 0, $lang) . $newData . "\0";
+                                } else {
+                                    $newData = pack('nn', length($newData), $lang) . $newData;
+                                }
+                            } elsif (not $$tagInfo{Format} or $$tagInfo{Format} =~ /^string/ and
+                                    not $$tagInfo{Binary} and not $$tagInfo{ValueConv})
+                            {
+                                # write all strings as UTF-8
+                                $newData = $et->Encode($newData, 'UTF8');
                             }
                         }
+                        $$didTag{$nvHash} = 1;   # set flag so we don't add this tag again
                     }
                 }
             }
@@ -1221,7 +1261,7 @@ sub WriteQuickTime($$$)
     }
     $et->VPrint(0, "  [deleting $delCount $dirName tag".($delCount==1 ? '' : 's')."]\n") if $delCount;
 
-    undef $createKeys unless $$addDirs{Keys};   # (Keys may have been written)
+    $createKeys &= ~0x01 unless $$addDirs{Keys};   # (Keys may have been written)
 
     # add new directories/tags at this level if necessary
     if ($canCreate and (exists $$et{EDIT_DIRS}{$dirName} or $createKeys)) {
@@ -1287,8 +1327,10 @@ sub WriteQuickTime($$$)
                 }
                 if ($$dirInfo{HasData}) {
                     # add 'data' header
+                    $lang or $lang = $undLang;
                     $newVal = pack('Na4Nnn',16+length($newVal),'data',$flags,$ctry,$lang).$newVal;
                 } elsif ($tag =~ /^\xa9/ or $$tagInfo{IText}) {
+                    $lang or $lang = $undLang;
                     if ($ctry) {
                         my $grp = $et->GetGroup($tagInfo,1);
                         $et->Warn("Can't use country code for $grp:$$tagInfo{Name}");
@@ -1696,7 +1738,7 @@ sub WriteMOV($$)
         $ftype = 'MOV';
     }
     $et->SetFileType($ftype); # need to set "FileType" tag for a Condition
-    $et->InitWriteDirs($dirMap{$ftype}, 'XMP');
+    $et->InitWriteDirs($dirMap{$ftype}, 'XMP', 'QuickTime');
     $$et{DirMap} = $dirMap{$ftype};     # need access to directory map when writing
     # track tags globally to avoid creating multiple tags in the case of duplicate directories
     $$et{DidTag} = { };
